/*
 Copyright (c) 2002 Information Equipment co.,LTD.
 All Right Reserved.

 Code by JaeHyuk Cho <minzkn@infoeq.co.kr> 
*/

#if !defined(DEF_sound_c)
#define DEF_sound_c "sound.c"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <linux/soundcard.h>

int MZ_SetupDSP(int s_Handle, int s_Channels, int s_Freq);
void MZ_Beep(int s_Second, int s_Freq);
int main(int s_Argc, char **s_Argv);

int MZ_SetupDSP(int s_Handle, int s_Channels, int s_Freq)
{
 int s_Return = (-1);	
 int s_Format;
 s_Format = AFMT_S16_NE;
 if(ioctl(s_Handle, SNDCTL_DSP_SETFMT, &s_Format) != (-1))
 {
  if(s_Format == AFMT_S16_NE)
  {
   if(ioctl(s_Handle, SNDCTL_DSP_CHANNELS, &s_Channels) != (-1))
   {
    if(ioctl(s_Handle, SNDCTL_DSP_SPEED, &s_Freq) != -1)s_Return = s_Freq;
    else fprintf(stderr, "sound.c: MZ_SetupDSP - [ERROR] Can not ioctl SPEED !!!\n");
   }
   else fprintf(stderr, "sound.c: MZ_SetupDSP - [ERROR] Can not ioctl CHANNEL !!!\n");
  }
  else fprintf(stderr, "sound.c: MZ_SetupDSP - [ERROR] s_Format is not support AFMT_S16_NE !!!\n");
 }
 else fprintf(stderr, "sound.c: MZ_SetupDSP - [ERROR] Can not ioctl DSP_SETFMT !!!\n");
 return(s_Return);
}

void MZ_Beep(int s_Second, int s_Freq)
{
 short *s_Sample;
 int s_Handle;
 s_Handle = open("/dev/dsp", O_RDWR);
 if(s_Handle >= 0)
 {
  MZ_SetupDSP(s_Handle, 2, 44100);	 
  s_Sample = (short *)malloc((44100 << 1) * sizeof(short));
  if(s_Sample)
  {
   double s_PI = 0.0, s_Grid;
   int s_Index = 0, s_Level;
   s_Grid = ((double)s_Freq / 44100.0) * (2.0 * M_PI);
   s_Level = (32768 >> 1) - 1;
   for(s_Index = 0;s_Index < 44100;s_Index++)
   {
    *(s_Sample + (s_Index << 1) + 0) = (short)(sin(s_PI) * (double)(s_Level));
    *(s_Sample + (s_Index << 1) + 1) = (short)(-sin(s_PI) * (double)(s_Level));
    s_PI += s_Grid;
    if(s_PI >= (2.0 * M_PI))s_PI = 0.0;
   }
   while(s_Second--)write(s_Handle, (void *)s_Sample, (44100 << 1) * sizeof(short)); 
   free(s_Sample);
  }
  else fprintf(stderr, "%s: %s - [ERROR] s_Sample is null !!!\n", __FILE__, __FUNCTION__);
  close(s_Handle);
 }
 else fprintf(stderr, "%s: %s - [ERROR] Can not open dsp !!!\n", __FILE__, __FUNCTION__);
}

int main(int s_Argc, char **s_Argv)
{
 if(s_Argc >= 3)
 {
  int s_PlayTime;
  int s_Frequency;
  sscanf(s_Argv[1], "%i", &s_PlayTime);
  sscanf(s_Argv[2], "%i", &s_Frequency);
  MZ_Beep(s_PlayTime, s_Frequency /* Hz */);
 }
 else
 {
  fprintf(stdout, "usage: %s <Play second> <Frequency>\n", s_Argv[0]);
  fprintf(stdout, "\tExample: bash# %s 1 1000\n", s_Argv[0]);
 } 
 return(0);	
}

#endif 

/* End of source */ 
