/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_memory_c 
#define DEF_SOURCE_memory_c "memory.c"

#include <string.h>

#include "mz.h"

int MZ_PeekByte(const void *s_From, int s_Offset);
int MZ_PeekWord(const void *s_From, int s_Offset);
long MZ_PeekDoubleWord(const void *s_From, int s_Offset);
void MZ_PokeByte(void *s_To, int s_Offset, int s_Value);
void MZ_PokeWord(void *s_To, int s_Offset, int s_Value);
void MZ_PokeDoubleWord(void *s_To, int s_Offset, long s_Value);

void MZ_ZeroMemory(void *s_To, int s_Size);
void MZ_SetMemory(void *s_To, int s_Value, int s_Size);
void MZ_CopyMemory(void *s_To, const void *s_From, int s_Size);
void MZ_MoveMemory(void *s_To, const void *s_From, int s_Size);

/* [ Static function ] ==================================================================== */

/* [ Public function ] ==================================================================== */
int MZ_PeekByte(const void *s_From, int s_Offset)
{
 return((int)(*((unsigned char *)__MZ_PEEK__(s_From, s_Offset))));
}

int MZ_PeekWord(const void *s_From, int s_Offset)
{
 return((int)(*((unsigned short *)__MZ_PEEK__(s_From, s_Offset))));
}

long MZ_PeekDoubleWord(const void *s_From, int s_Offset)
{
 return((long)(*((unsigned long *)__MZ_PEEK__(s_From, s_Offset))));
}

void MZ_PokeByte(void *s_To, int s_Offset, int s_Value)
{
 *((unsigned char *)__MZ_PEEK__(s_To, s_Offset)) = (unsigned char)s_Value;
}
void MZ_PokeWord(void *s_To, int s_Offset, int s_Value)
{
 *((unsigned short *)__MZ_PEEK__(s_To, s_Offset)) = (unsigned short)s_Value;
}

void MZ_PokeDoubleWord(void *s_To, int s_Offset, long s_Value)
{
 *((unsigned long *)__MZ_PEEK__(s_To, s_Offset)) = (unsigned long)s_Value;
}

void MZ_ZeroMemory(void *s_To, int s_Size)
{
#ifdef WIN32
 MZ_SetMemory(s_To, 0, s_Size);
#else
 __asm__ volatile(
  "\n\t"
  "cld\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "stosb %%al, %%es:(%0)\n\t"
  "0:\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "stosw %%ax, %%es:(%0)\n\t"
  "0:\n\t"
  "repz stosl %1, %%es:(%0)\n\t"
  "\n\t"
  :
  : "D"(s_To), "a"(0), "c"(s_Size)
 );
#endif
}

void MZ_SetMemory(void *s_To, int s_Value, int s_Size)
{
#ifdef WIN32
 (void)memset(s_To, s_Value, (size_t)s_Size);
#else
 /* %edx private backup %eax */
 __asm__ volatile(
  "\n\t"
  "movb %%al, %%ah\n\t"
  "movl %1, %%edx\n\t"
  "shll $16, %1\n\t"
  "movw %%dx, %%ax\n\t"
  "cld\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "stosb %%al, %%es:(%0)\n\t"
  "0:\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "stosw %%ax, %%es:(%0)\n\t"
  "0:\n\t"
  "repz stosl %1, %%es:(%0)\n\t"
  "\n\t"
  :
  : "D"(s_To), "a"(s_Value), "c"(s_Size)
 );
#endif
}

void MZ_CopyMemory(void *s_To, const void *s_From, int s_Size)
{
#ifdef WIN32
 (void)memcpy(s_To, s_From, (size_t)s_Size);
#else
 __asm__ volatile(
  "\n\t"
  "cld\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "movsb %%ds:(%1), %%es:(%0)\n\t"
  "0:\n\t"
  "shrl $1, %2\n\t"
  "jnc 0f\n\t"
  "movsw %%ds:(%1), %%es:(%0)\n\t"
  "0:\n\t"
  "repz movsl %%ds:(%1), %%es:(%0)\n\t"
  "\n\t"
  :
  : "D"(s_To), "S"(s_From), "c"(s_Size)
 );
#endif
}

void MZ_MoveMemory(void *s_To, const void *s_From, int s_Size)
{
#ifdef WIN32
 (void)memmove(s_To, s_From, (size_t)s_Size);
#else
 __asm__ volatile(
  "\n\t"
  "cld\n\t"
  "repz movsb %%ds:(%1), %%es:(%0)\n\t"
  "\n\t"
  :
  : "D"(s_To), "S"(s_From), "c"(s_Size)
 );
#endif
}

#endif

/* End of source */
