/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_HEADER_mz_h 
#define DEF_HEADER_mz_h "mz.h"

#include <sys/types.h>
#include <stdarg.h>

#ifndef DEF_MACRO_PEEK
#define DEF_MACRO_PEEK (1)
#define __MZ_PEEK__(s_From, s_Offset) (((unsigned char *)(s_From)) + (s_Offset))
#define __MZ_PEEK_R__(s_From, s_Offset) (((unsigned char *)(s_From)) - (s_Offset))
#endif

#define DEF_MIXER_GET  (0)
#define DEF_MIXER_SET  (1)
#define DEF_MIXER_KEEP (-1)

/* - */
#ifndef DEF_STRUCT_BUFFER
#define DEF_STRUCT_BUFFER (1)
typedef struct ts_MZ_Buffer_Node
{
 struct ts_MZ_Buffer_Node *Next;
 void *Data;
 int Index, PopIndex, Size;
}t_MZ_Buffer_Node;

typedef struct ts_MZ_Buffer
{
 t_MZ_Buffer_Node *Node, *NewNode, *LastNode;
 int Size;
}t_MZ_Buffer;
#endif

/* - */
typedef enum
{
 E_TAR_ERROR_NONE                  = 0x00000000,
 E_TAR_ERROR_FOUND                 = 0x00000001,
 E_TAR_ERROR_CREATE                = 0x00000002,
 E_TAR_ERROR_PERMISSION            = 0x00000004,
 E_TAR_ERROR_INVALID_OPERATION     = 0x00000008,
 E_TAR_ERROR_INVALID_DATA          = 0x00000010,
 E_TAR_ERROR_MEMORY                = 0x00000020,
}e_TAR_ERROR;

typedef enum
{
 E_TAR_FLAG_NONE                   = 0x00000000,	
 E_TAR_FLAG_DEBUG                  = 0x00000001,
 E_TAR_FLAG_VERBOSE                = 0x00000002,
 E_TAR_FLAG_LIST                   = 0x00000004,
 E_TAR_FLAG_EXTRACT                = 0x00000008,
 E_TAR_FLAG_CREATE                 = 0x00000010,
 E_TAR_FLAG_STREAM                 = 0x00000020,
}e_TAR_FLAG;

typedef enum
{
 E_TAR_HEADER_SIZE_NAME            = 100,
 E_TAR_HEADER_SIZE_MODE            =   8,
 E_TAR_HEADER_SIZE_UID             =   8,
 E_TAR_HEADER_SIZE_GID             =   8,
 E_TAR_HEADER_SIZE_SIZE            =  12,
 E_TAR_HEADER_SIZE_MAKETIME        =  12,
 E_TAR_HEADER_SIZE_CHECKSUM        =   8,
 E_TAR_HEADER_SIZE_TYPEFLAG        =   1,
 E_TAR_HEADER_SIZE_LINKNAME        = 100,
 E_TAR_HEADER_SIZE_MAGIC           =   6,
 E_TAR_HEADER_SIZE_VERSION         =   2,
 E_TAR_HEADER_SIZE_UNAME           =  32,
 E_TAR_HEADER_SIZE_GNAME           =  32,
 E_TAR_HEADER_SIZE_DEVMAJOR        =   8,
 E_TAR_HEADER_SIZE_DEVMINOR        =   8,
 E_TAR_HEADER_SIZE_PREFIX          = 155,
 E_TAR_HEADER_SIZE_PADDING         =  12,
}e_TAR_HEADER; 

typedef enum
{
 E_TAR_TYPEFLAG_REGULAR            = '0',  /* Normal regular file */
 E_TAR_TYPEFLAG_REGULAR_BUG        = '\0', /* For ancient bug complete */
 E_TAR_TYPEFLAG_LINK               = '1',  /* Hard link */
 E_TAR_TYPEFLAG_SYMLINK            = '2',  /* Symbolic link */
 E_TAR_TYPEFLAG_CHARDEV            = '3',  /* Character device */
 E_TAR_TYPEFLAG_BLOCKDEV           = '4',  /* Block device */
 E_TAR_TYPEFLAG_DIRECTORY          = '5',  /* Directory */
 E_TAR_TYPEFLAG_FIFO               = '6',  /* FIFO */
 E_TAR_TYPEFLAG_RESERVED           = '7',  /* Reserved */
 E_TAR_TYPEFLAG_LONGLINK           = 'K',  /* Long link name */
 E_TAR_TYPEFLAG_LONGNAME           = 'L',  /* Long file name */
}e_TAR_TYPEFLAG;

typedef struct
{
 char                       Name[E_TAR_HEADER_SIZE_NAME];
 char                       Mode[E_TAR_HEADER_SIZE_MODE];
 char                       UID[E_TAR_HEADER_SIZE_UID];
 char                       GID[E_TAR_HEADER_SIZE_GID];
 char                       Size[E_TAR_HEADER_SIZE_SIZE];
 char                       MakeTime[E_TAR_HEADER_SIZE_MAKETIME];
 char                       CheckSum[E_TAR_HEADER_SIZE_CHECKSUM];
 char                       TypeFlag[E_TAR_HEADER_SIZE_TYPEFLAG];
 char                       LinkName[E_TAR_HEADER_SIZE_LINKNAME];
 char                       Magic[E_TAR_HEADER_SIZE_MAGIC];
 char                       Version[E_TAR_HEADER_SIZE_VERSION];
 char                       UName[E_TAR_HEADER_SIZE_UNAME];
 char                       GName[E_TAR_HEADER_SIZE_GNAME];
 char                       DevMajor[E_TAR_HEADER_SIZE_DEVMAJOR];
 char                       DevMinor[E_TAR_HEADER_SIZE_DEVMINOR];
 char                       Prefix[E_TAR_HEADER_SIZE_PREFIX];
 char                       Padding[E_TAR_HEADER_SIZE_PADDING]; 
}t_TAR_HEADER; /* Total header size is 512 bytes */

typedef struct ts_TAR_List 
{
 struct ts_TAR_List        *Next;	
 char                      *Name;
 unsigned long              Mode;
 uid_t                      UID;
 gid_t                      GID;
 unsigned long long         Size;
 time_t                     MakeTime;
 long                       CheckSum;
 e_TAR_TYPEFLAG             TypeFlag;
 char                      *LinkName;
 char                      *UName;
 char                      *GName;
 unsigned long              DevMajor;
 unsigned long              DevMinor;
 unsigned long long         Position;
}t_TAR_List;

typedef struct
{
 char                      *TarName;	
 t_TAR_List                *List;
 unsigned long              ErrorCode;
 unsigned long              Flag;
 int                        ReadHandle;
 int                        WriteHandle;
}t_TAR;

/* - */
typedef struct 
{
 unsigned long        State[4];
 unsigned long        Count[2];
 unsigned char        Buffer[64];
}t_MD5_Context;

/* - */
typedef struct ts_MZ_PortScan
{
 struct ts_MZ_PortScan *Next;
 int Port;
}t_MZ_PortScan;

/* - */
typedef struct ts_MZ_Node_Pointer
{
 struct ts_MZ_Node_Pointer *Next;
 void                      *Pointer;
}t_MZ_Node_Pointer;

/* - */
typedef struct ts_MZ_Parser
{
 t_MZ_Buffer       *FIFO;
 t_MZ_Node_Pointer *Line;
 int                IsThis, IsMSB, IsLF, IsCR, LineCount;
}t_MZ_Parser;

/* - */
typedef struct ts_MZ_NetworkRoute
{
 struct ts_MZ_NetworkRoute     *Next;
 char                          *Name;
 char                          *Destination, *Gateway, *Mask;
 int                            Flags, RefCnt, Use, Metric;
}t_MZ_NetworkRoute;

typedef struct ts_MZ_NetworkInterface
{
 struct ts_MZ_NetworkInterface *Next;
 char                          *Name;
 char                          *IP, *Broadcast, *Netmask, *MAC;
 int                            Flags, MTU, Metric;
}t_MZ_NetworkInterface;

typedef struct ts_MZ_NetworkInfo
{
 t_MZ_NetworkRoute        *Route;
 t_MZ_NetworkInterface    *Interface;
}t_MZ_NetworkInfo;

#ifdef __cplusplus
extern "C"{
#endif

#ifndef DEF_SOURCE_release_c
extern int MZ_Release(void);
extern const char *MZ_ReleaseString(void);
#endif

#ifndef DEF_SOURCE_memory_c
#define DEF_FUNCTION_MEMORY
extern int MZ_PeekByte(const void *s_From, int s_Offset);
extern int MZ_PeekWord(const void *s_From, int s_Offset);
extern long MZ_PeekDoubleWord(const void *s_From, int s_Offset);
extern void MZ_PokeByte(void *s_To, int s_Offset, int s_Value);
extern void MZ_PokeWord(void *s_To, int s_Offset, int s_Value);
extern void MZ_PokeDoubleWord(void *s_To, int s_Offset, long s_Value);

extern void MZ_ZeroMemory(void *s_To, int s_Size);
extern void MZ_SetMemory(void *s_To, int s_Value, int s_Size);
extern void MZ_CopyMemory(void *s_To, const void *s_From, int s_Size);
extern void MZ_MoveMemory(void *s_To, const void *s_From, int s_Size);
#endif

#ifndef DEF_SOURCE_string_c
extern int MZ_StringLength(const void *s_String);
extern void *MZ_AllocString(const void *s_String, int s_Size);
extern void *MZ_VSPrintf(const void *s_Format, va_list s_VaArg);
extern void *MZ_SPrintf(const void *s_Format, ...);
extern char *MZ_SearchString(const void *s_String, const void *s_This);
extern char *MZ_StripString(const void *s_String, const void *s_StripString, int s_Flag);
#endif

#ifndef DEF_SOURCE_atomic_c
extern void MZ_AtomicAdd(int * volatile s_To, int s_Value);
extern void MZ_AtomicSub(int * volatile s_From, int s_Value);
extern int MZ_AtomicExchange(int * volatile s_To, int s_Value);
extern int MZ_InitLock(int * volatile s_Key);
extern void MZ_Lock(int * volatile s_Key);
extern void MZ_UnLock(int * volatile s_Key);
#endif

#ifndef DEF_SOURCE_hook_c
#define DEF_FUNCTION_ALLOC
extern void *MZ_Alloc(int s_AllocateSize);
extern void MZ_Free(void *s_AllocMemory);
extern int MZ_AllocSize(void);
extern int MZ_AllocCount(void);
#endif

#ifndef DEF_SOURCE_buffer_c
extern t_MZ_Buffer *MZ_CreateBuffer(void);
extern t_MZ_Buffer_Node *MZ_DestroyBuffer_Node(t_MZ_Buffer_Node *s_Buffer_Node);
extern t_MZ_Buffer *MZ_DestroyBuffer(t_MZ_Buffer *s_Buffer);
extern int MZ_CheckBuffer(t_MZ_Buffer *s_Buffer);
extern int MZ_LimitCheckBuffer(t_MZ_Buffer *s_Buffer, int s_LimitSize);
extern t_MZ_Buffer *MZ_PushBuffer(t_MZ_Buffer *s_Buffer, const void *s_Data, int s_Size);
extern t_MZ_Buffer *MZ_PassPushBuffer(t_MZ_Buffer *s_Buffer, void *s_Data, int s_Size);
extern int MZ_PopBuffer(t_MZ_Buffer *s_Buffer, void *s_Data, int s_Size);
extern int MZ_PrePopBuffer(t_MZ_Buffer *s_Buffer, void *s_Data, int s_Size, int s_Offset);
extern void *MZ_PassPopBuffer(t_MZ_Buffer *s_Buffer, int *s_Size);
extern t_MZ_Buffer *MZ_FlushBuffer(t_MZ_Buffer *s_Buffer);
#endif

#ifndef DEF_SOURCE_mixer_c
/*
 ==================================================
  Mixer argument
 ==================================================

 - See also /usr/include/linux/soundcard.h

 s_DeviceName   = NULL(g_DefaultMixerDeviceName = "/dev/mixer")
 
 s_Channel      = SOUND_MIXER_{CHANNEL_NAME}
 s_ChannelMask  = SOUND_MASK_{CHANNEL_NAME}

 {CHANNEL_NAME} = VOLUME, BASS, TREBLE, SYNTH, 
                  PCM, SPEAKER, LINE, LINE, 
                  MIC, CD, IMIX, ALTPCM, 
                  RECLEV, IGAIN, OGAIN, LINE1, 
                  LINE2, LINE3, DIGITAL1, DIGITAL2, 
                  DIGITAL3, PHONEIN, PHONEOUT, RADIO, 
                  VIDEO, MONITOR, ...

 DEF_MIXER_KEEP = Keep old value	  
		  
 s_Left         = 0 ~ 100 | DEF_MIXER_KEEP
 s_Right        = 0 ~ 100 | DEF_MIXER_KEEP
 s_Mute         = 0 | 1 | DEF_MIXER_KEEP
*/

extern int MZ_GetMixer(const char *s_DeviceName, int s_Channel, int *s_Left, int *s_Right, int *s_Mute);
extern int MZ_GetVolume(const char *s_DeviceName, int s_Channel, int *s_Left, int *s_Right);
extern int MZ_GetMuteStatus(const char *s_DeviceName, int s_Channel);
extern int MZ_GetMute(const char *s_DeviceName, int s_Channel);
extern int MZ_SetMixer(const char *s_DeviceName, int s_Channel, int s_Left, int s_Right, int s_Mute);
extern int MZ_SetVolume(const char *s_DeviceName, int s_Channel, int s_Left, int s_Right);
extern int MZ_SetMuteStatus(const char *s_DeviceName, int s_Channel, int s_Mute);
extern int MZ_SetMute(const char *s_DeviceName, int s_Channel, int s_Mute);
extern int MZ_SetMixerMask(const char *s_DeviceName, int s_ChannelMask, int s_Left, int s_Right, int s_Mute);
extern int MZ_SetVolumeMask(const char *s_DeviceName, int s_ChannelMask, int s_Left, int s_Right);
extern int MZ_SetMuteStatusMask(const char *s_DeviceName, int s_ChannelMask, int s_Mute);
extern int MZ_SetMuteMask(const char *s_DeviceName, int s_ChannelMask, int s_Mute);

extern const int c_SoundDevices;       /* Set SOUND_MIXER_NRDEVICES */
extern const char *c_MixerName[];      /* Set SOUND_DEVICE_NAMES */ 
extern char *g_DefaultMixerDeviceName; /* Default "/dev/mixer" */
#endif

#ifndef DEF_SOURCE_dsp_c
/*
 ==================================================
  DSP argument
 ==================================================

 - See also /usr/include/linux/soundcard.h

 s_Format       = AFMT_QUERY, AFMT_MU_LAW, AFMT_A_LAW, AFMT_IMA_ADPCM,
                  AFMT_U8, AFMT_S16_LE, AFMT_S16_BE, AFMT_S8, 
		  AFMT_U16_LE, AFMT_U16_BE, AFMT_MPEG, AFMT_AC3, 
		  AFMT_S16_NE, ...
 s_Channels     = 1, 2, ...
 s_Frequency    = 8000, 12000, 11025, 16000, 24000, 22050, 32000, 44100, 48000, ... (Sound card capability) 

 NOTE: DSP write block is 8KByte normal. (My test result...) 
*/

extern int MZ_DSP_SetupFormat(int s_Handle, int s_Format);
extern int MZ_DSP_SetupChannels(int s_Handle, int s_Channels);
extern int MZ_DSP_SetupFrequency(int s_Handle, int s_Frequency);
#endif

#ifndef DEF_SOURCE_framebuffer_c
#endif

#ifndef DEF_SOURCE_keyboard_c
#endif

#ifndef DEF_SOURCE_mouse_c
#endif

#ifndef DEF_SOURCE_serial_c
#endif

#ifndef DEF_SOURCE_rdate_c
extern time_t MZ_GetRemoteTimeTCP(const char *s_RemoteIP, int s_RemotePort, int s_TimeOut);
extern time_t MZ_GetRemoteTimeUDP(const char *s_RemoteIP, int s_RemotePort, int s_TimeOut, int s_TTL);
#endif

#ifndef DEF_SOURCE_tar_c
extern t_TAR *MZ_OpenTAR(char *s_TarName, unsigned long s_Flag);
extern t_TAR *MZ_CloseTAR(t_TAR *s_HANDLE_TAR);
extern t_TAR *MZ_ListTAR(t_TAR *s_HANDLE_TAR);
extern unsigned long MZ_DoTAR(t_TAR *s_Tar, t_TAR_List *s_TarList, unsigned long s_Flag);
#endif

#ifndef DEF_SOURCE_crc32_c
extern unsigned long MZ_CRC32_Init(void);
extern unsigned long MZ_CRC32_Make(unsigned long s_Seed, const void *s_Data, int s_Length);
#endif

#ifndef DEF_SOURCE_md5_c
extern void MZ_MD5_Init(t_MD5_Context *s_Context);
extern void MZ_MD5_Update(t_MD5_Context *s_Context, const void *s_Buffer, unsigned long s_Length);
extern void MZ_MD5_TransForm(void *s_Buffer, void *s_In);
extern void MZ_MD5_Final(t_MD5_Context *s_Context, void *s_Digest);
#endif

#ifndef DEF_SOURCE_portscan_c
extern t_MZ_PortScan *MZ_PortScan(const char *s_RemoteIP, int s_StartPort, int s_EndPort, int s_TimeOut);
extern t_MZ_PortScan *MZ_FreePortScan(t_MZ_PortScan *s_PortScan);
#endif

#ifndef DEF_SOURCE_screen_c
extern void MZ_SCREEN_Clear(void);
extern void MZ_SCREEN_MoveCursor(int s_x, int s_y);
#endif

#ifndef DEF_SOURCE_node_c
extern t_MZ_Node_Pointer *MZ_Node_Pointer(t_MZ_Node_Pointer *s_Handle, void *s_Pointer, t_MZ_Node_Pointer **s_Reference);
extern t_MZ_Node_Pointer *MZ_DestroyNode_Pointer(t_MZ_Node_Pointer *s_Handle, int s_PointerFree);
#endif

#ifndef DEF_SOURCE_parser_c
extern t_MZ_Parser *MZ_Parser_Line(t_MZ_Parser *s_Handle, const void *s_Data, int s_Size);
extern t_MZ_Parser *MZ_DestroyParser(t_MZ_Parser *s_Handle);
#endif

#ifndef DEF_SOURCE_debug_c
#endif

#ifndef DEF_SOURCE_dump_c
extern void MZ_DumpEx(int s_Offset, const void *s_Data, int s_Size);
extern int MZ_DumpFile(const char *s_FileName, int s_Offset, int s_Size);
extern void MZ_Dump(const void *s_Data, int s_Size);
#endif

#ifndef DEF_SOURCE_registry_c
#endif

#ifndef DEF_SOURCE_network_c
extern int MZ_OpenSocket_TCP(void);
extern int MZ_OpenSocket_UDP(void);
extern int MZ_OpenSocket_UDS(void);
extern int MZ_CloseSocket(int s_Socket);
extern int MZ_SetupInAddress(void *s_SockAddress, int s_Family, const char *s_IP, int s_Port);
extern int MZ_BindSocket(int s_Socket, void *s_BindAddress);
extern int MZ_ListenSocket(int s_Socket, int s_BackLog);
extern int MZ_ConnectSocket(int s_Socket, void *s_RemoteAddress, int s_TimeOut);
extern int MZ_SetupTTL(int s_Socket, int s_TTL);
#endif

#ifndef DEF_SOURCE_networkinfo_c
extern t_MZ_NetworkInfo *MZ_GetNetworkInfo(t_MZ_NetworkInfo *s_NetworkInfo);
extern t_MZ_NetworkInfo *MZ_DestroyNetworkInfo(t_MZ_NetworkInfo *s_NetworkInfo);
#endif

#ifndef DEF_SOURCE_container_c
#endif

#ifndef DEF_SOURCE_signal_c
#endif

#ifndef DEF_SOURCE_thread_c
#endif

#ifdef __cplusplus
}
#endif

#endif

/* End of header */
