/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_parser_c 
#define DEF_SOURCE_parser_c "parser.c"

#include <stdio.h>

#include "mz.h"

static t_MZ_Parser *__MZ_CreateParser__(void);
static t_MZ_Parser *__MZ_DestroyParser__(t_MZ_Parser *s_Handle);

t_MZ_Parser *MZ_Parser_Line(t_MZ_Parser *s_Handle, const void *s_Data, int s_Size);
t_MZ_Parser *MZ_DestroyParser(t_MZ_Parser *s_Handle);

/* [ Static function ] ==================================================================== */
static t_MZ_Parser *__MZ_CreateParser__(void)
{
 t_MZ_Parser *s_Return;
 s_Return = (t_MZ_Parser *)MZ_Alloc(sizeof(t_MZ_Parser));
 if(s_Return)
 {
  s_Return->FIFO = (t_MZ_Buffer *)0;
  s_Return->Line = (t_MZ_Node_Pointer *)0;
  s_Return->IsThis = s_Return->IsMSB = 0;
  s_Return->IsCR = s_Return->IsLF = s_Return->LineCount = 0;
 }
 return(s_Return);
}

static t_MZ_Parser *__MZ_DestroyParser__(t_MZ_Parser *s_Handle)
{
 if(s_Handle)
 {
  if(s_Handle->FIFO)s_Handle->FIFO = MZ_DestroyBuffer(s_Handle->FIFO);
  if(s_Handle->Line)s_Handle->Line = MZ_DestroyNode_Pointer(s_Handle->Line, 1/* Free */);
  MZ_Free(s_Handle); 
  s_Handle = (t_MZ_Parser *)0;
 }
 return(s_Handle);
}

/* [ Public function ] ==================================================================== */
t_MZ_Parser *MZ_Parser_Line(t_MZ_Parser *s_Handle, const void *s_Data, int s_Size)
{
 int s_Offset, s_Index, s_CheckBuffer, s_Byte;
 void *s_Line;
 t_MZ_Node_Pointer *s_Reference = (t_MZ_Node_Pointer *)0;
 if(s_Handle == (t_MZ_Parser *)0)s_Handle = __MZ_CreateParser__();
 if(s_Handle)
 {
  if(s_Data && s_Size > 0)
  {
   s_Offset = s_Index = 0;
   while(s_Offset < s_Size)
   {
    s_Byte = MZ_PeekByte(s_Data, s_Offset);
    switch(s_Byte)
    {
     case '\r' /* 0x0d */: 
	  if((s_Byte & 0x80) == 0)
	  {
	   if(s_Handle->IsCR == 0 && s_Handle->IsLF == 0)s_Handle->IsThis = 1;
	   s_Handle->IsCR++; 
	   break;
	  }
     case '\n' /* 0x0a */:
	  if((s_Byte & 0x80) == 0)
	  {
	   if(s_Handle->IsCR == 0)s_Handle->IsThis = 1;
	   if(s_Handle->IsLF > 0)s_Index = s_Offset;
	   s_Handle->IsLF++; 
	   break;
	  }
     default: 
	  if(s_Handle->IsCR > 0 || s_Handle->IsLF > 0)
	  {
	   s_Index = s_Offset; 
	   s_Handle->IsCR = s_Handle->IsLF = 0; 
	  }
	  if(s_Byte & 0x80)
	  {
	   if(s_Handle->IsMSB == 0)s_Handle->IsMSB = 1;
	   else s_Handle->IsMSB = 0;
	  }
	  else s_Handle->IsMSB = 0;
	  break; 
    }
    if(s_Handle->IsThis == 1)
    {
     s_CheckBuffer = MZ_CheckBuffer(s_Handle->FIFO);
     if(s_CheckBuffer > 0)
     {
      s_Line = MZ_Alloc(s_CheckBuffer + (s_Offset - s_Index) + 1);
      (void)MZ_PopBuffer(s_Handle->FIFO, s_Line, s_CheckBuffer);
      if(s_Offset != s_Index)MZ_CopyMemory(__MZ_PEEK__(s_Line, s_CheckBuffer), __MZ_PEEK__(s_Data, s_Index), s_Offset - s_Index);
      MZ_PokeByte(s_Line, s_CheckBuffer + (s_Offset - s_Index), '\0'); 
     }
     else if(s_Offset != s_Index)s_Line = MZ_AllocString(__MZ_PEEK__(s_Data, s_Index), s_Offset - s_Index);
     else s_Line = (void *)0;
     s_Handle->Line = MZ_Node_Pointer(s_Handle->Line, s_Line, &s_Reference);
     s_Handle->LineCount++;
     s_Index = s_Offset + 1;
     s_Handle->IsThis = 0;
    }
    s_Offset++;
   }
   if(s_Offset != s_Index)s_Handle->FIFO = MZ_PushBuffer(s_Handle->FIFO, __MZ_PEEK__(s_Data, s_Index), s_Offset - s_Index);
  }
  else if(s_Handle->FIFO)
  { /* Flush FIFO */
   s_CheckBuffer = MZ_CheckBuffer(s_Handle->FIFO);
   if(s_CheckBuffer > 0)
   {
    s_Line = MZ_Alloc(s_CheckBuffer + 1);
    (void)MZ_PopBuffer(s_Handle->FIFO, s_Line, s_CheckBuffer);
    MZ_PokeByte(s_Line, s_CheckBuffer, '\0'); 
    s_Handle->Line = MZ_Node_Pointer(s_Handle->Line, s_Line, &s_Reference);
    s_Handle->LineCount++;
   } 
   s_Handle->FIFO = MZ_DestroyBuffer(s_Handle->FIFO);
  }
 }
 return(s_Handle);
}

t_MZ_Parser *MZ_DestroyParser(t_MZ_Parser *s_Handle)
{
 return(__MZ_DestroyParser__(s_Handle));
}

#endif

/* End of source */
