/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_portscan_c 
#define DEF_SOURCE_portscan_c "portscan.c"

#ifndef DEF_STANDALONE 
#define DEF_STANDALONE (0)
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/select.h>

#if DEF_STANDALONE == (0)
#include "mz.h"
#else
#include <stdio.h>

typedef struct ts_MZ_PortScan
{
 struct ts_MZ_PortScan *Next;
 int Port;
}t_MZ_PortScan;

#define MZ_Alloc malloc
#define MZ_Free free
#endif

typedef struct ts_MZ_PortSelect
{
 struct ts_MZ_PortSelect *Next;
 int                      Socket, Port, IsActive;
}t_MZ_PortSelect;

t_MZ_PortScan *MZ_PortScan(const char *s_RemoteIP, int s_StartPort, int s_EndPort, int s_TimeOut);
t_MZ_PortScan *MZ_FreePortScan(t_MZ_PortScan *s_PortScan);

/* [ Static function ] ==================================================================== */

/* [ Public function ] ==================================================================== */
t_MZ_PortScan *MZ_PortScan(const char *s_RemoteIP, int s_StartPort, int s_EndPort, int s_TimeOut)
{
 const char c_Protocol[] = "tcp";
 const int c_MaxSelect = FD_SETSIZE;
 t_MZ_PortScan *s_Return = (t_MZ_PortScan *)0;
 t_MZ_PortSelect *s_PortSelect = (t_MZ_PortSelect *)0, *s_NewPort, *s_TracePort;
 struct hostent *s_HostEntry;
 fd_set s_FD_Read, s_FD_Write;
 struct timeval s_TimeVal;
 int s_SocketError, s_SocketErrorLength, s_Protocol, s_Index, s_Flags, s_MaxFD, s_IsSelect;
 struct protoent *s_ProtoEntry;
 struct sockaddr_in s_BindAddress, s_RemoteAddress;
 socklen_t s_BindAddressLength, s_RemoteAddressLength;
 if(s_RemoteIP)
 {
  if(s_StartPort <= 0)s_StartPort = 1;
  if(s_EndPort < s_StartPort)
  {
   if(0xffff < s_StartPort)s_EndPort = s_StartPort;
   else s_EndPort = 0xffff;
  }
  if((s_EndPort - s_StartPort + 1) > c_MaxSelect)
  { /* Block select */
   s_Return = MZ_PortScan(s_RemoteIP, s_StartPort + c_MaxSelect, s_EndPort, s_TimeOut);
   s_EndPort = s_StartPort + c_MaxSelect - 1; 
  }
  s_BindAddress.sin_family = AF_INET;
  s_BindAddress.sin_addr.s_addr = htonl(INADDR_ANY);
  s_BindAddress.sin_port = htons(0);
#if DEF_STANDALONE == (0)
  MZ_ZeroMemory((void *)(&s_BindAddress.sin_zero[0]), sizeof(s_BindAddress.sin_zero));
#else
  memset((void *)(&s_BindAddress.sin_zero[0]), 0, sizeof(s_BindAddress.sin_zero));
#endif
  s_BindAddressLength = sizeof(s_BindAddress);
  s_RemoteAddress.sin_family = AF_INET;
  s_HostEntry = gethostbyname(s_RemoteIP);
  if(s_HostEntry)s_RemoteAddress.sin_addr.s_addr = *((unsigned long *)s_HostEntry->h_addr);
  else s_RemoteAddress.sin_addr.s_addr = htonl(INADDR_ANY); 
  s_RemoteAddress.sin_port = htons(0);
#if DEF_STANDALONE == (0)
  MZ_ZeroMemory((void *)(&s_RemoteAddress.sin_zero[0]), sizeof(s_RemoteAddress.sin_zero));
#else
  memset((void *)(&s_RemoteAddress.sin_zero[0]), 0, sizeof(s_RemoteAddress.sin_zero));
#endif
  s_RemoteAddressLength = sizeof(s_RemoteAddress);
  s_ProtoEntry = getprotobyname(c_Protocol);
  s_Protocol = s_ProtoEntry ? s_ProtoEntry->p_proto : 0;
  /* Select node */
  for(s_Index = s_StartPort;s_Index <= s_EndPort;s_Index++)
  {
   s_NewPort = (t_MZ_PortSelect *)MZ_Alloc(sizeof(t_MZ_PortSelect));
   if(s_NewPort)
   {
    s_NewPort->Next = (t_MZ_PortSelect *)0;
    s_NewPort->Port = s_Index;
    s_NewPort->IsActive = (-2);
    s_NewPort->Socket = (-1);
    if(s_PortSelect)
    {
     s_NewPort->Next = s_PortSelect;
     s_PortSelect = s_NewPort;
    }
    else s_PortSelect = s_NewPort;
   }
  }
  s_TimeVal.tv_sec = s_TimeOut, s_TimeVal.tv_usec = 0;
  do
  {
   s_MaxFD = (-1); 
   FD_ZERO(&s_FD_Read);
   FD_ZERO(&s_FD_Write);
   s_TracePort = s_PortSelect;
   while(s_TracePort)
   {
    if(s_TracePort->IsActive == (-2)) 
    { /* Try connect */
     s_TracePort->IsActive = (-1);
     s_TracePort->Socket = socket(AF_INET, SOCK_STREAM, s_Protocol);
     if(s_TracePort->Socket >= 0)
     {
      s_Flags = fcntl(s_TracePort->Socket, F_GETFL);
      if(fcntl(s_TracePort->Socket, F_SETFL, s_Flags | O_NONBLOCK) == 0)
      {
       if(bind(s_TracePort->Socket, (struct sockaddr *)(&s_BindAddress), s_BindAddressLength) == 0)
       {
	s_RemoteAddress.sin_port = htons(s_TracePort->Port);
        if(connect(s_TracePort->Socket, (struct sockaddr *)(&s_RemoteAddress), s_RemoteAddressLength) == 0)
	{
         s_TracePort->IsActive = 1;
	}
	else s_TracePort->IsActive = 0;
       }
      }
      if(s_TracePort->IsActive != 0)
      {
#ifdef WIN32
       closesocket(s_TracePort->Socket);
#else
       close(s_TracePort->Socket);
#endif
       s_TracePort->Socket = (-1);
      }
     }
    }
    if(s_TracePort->Socket >= 0 && s_TracePort->IsActive == 0)
    {
     if(s_MaxFD < s_TracePort->Socket)s_MaxFD = s_TracePort->Socket;
     FD_SET(s_TracePort->Socket, &s_FD_Read);
     FD_SET(s_TracePort->Socket, &s_FD_Write);
    }
    s_TracePort = s_TracePort->Next;
   }
   if(s_MaxFD >= 0)
   { 
    s_IsSelect = select(s_MaxFD + 1, (fd_set *)(&s_FD_Read), (fd_set *)(&s_FD_Write), (fd_set *)0, s_TimeOut > 0 ? (struct timeval *)(&s_TimeVal) : (struct timeval *)0);
    if(s_IsSelect > 0)
    {
     s_TracePort = s_PortSelect;
     while(s_TracePort)
     {
      if(s_TracePort->Socket >= 0 && s_TracePort->IsActive == 0)
      {
       if(FD_ISSET(s_TracePort->Socket, &s_FD_Read) || FD_ISSET(s_TracePort->Socket, &s_FD_Write))
       {
        s_SocketErrorLength = sizeof(s_SocketError);
	if(getsockopt(s_TracePort->Socket, SOL_SOCKET, SO_ERROR, &s_SocketError, &s_SocketErrorLength) == 0)
	{
	 if(s_SocketError == 0)s_TracePort->IsActive = 1;
	}
#ifdef WIN32
        closesocket(s_TracePort->Socket);
#else
        close(s_TracePort->Socket);
#endif
        s_TracePort->Socket = (-1);	
        break;
       }
      }
      s_TracePort = s_TracePort->Next;
     }
    }
    else break; /* Time out */ 
   }
   else break; /* Finish scan */
  }while(s_IsSelect > 0);
  /* Free node */
  while(s_PortSelect)
  {
   s_TracePort = s_PortSelect;
   s_PortSelect = s_PortSelect->Next;
   if(s_TracePort->Socket >= 0)close(s_TracePort->Socket);
   if(s_TracePort->IsActive == 1)
   {
    t_MZ_PortScan *s_New;
    s_New = (t_MZ_PortScan *)MZ_Alloc(sizeof(t_MZ_PortScan)); 
    if(s_New)
    {
     s_New->Next = (t_MZ_PortScan *)0;
     s_New->Port = s_TracePort->Port;
     if(s_Return)
     {
      s_New->Next = s_Return;
      s_Return = s_New;
     }
     else s_Return = s_New;
    }
   }
   MZ_Free(s_TracePort);
  }
 }
 return(s_Return);
}

t_MZ_PortScan *MZ_FreePortScan(t_MZ_PortScan *s_PortScan)
{
 t_MZ_PortScan *s_Trace;
 if(s_PortScan)
 {
  while(s_PortScan)
  {
   s_Trace = s_PortScan;
   s_PortScan = s_PortScan->Next;
   MZ_Free(s_Trace);
  }
 }
 return(s_PortScan);
}

#if DEF_STANDALONE == (1)
int main(int s_Argc, char *s_Argv[])
{
 int s_Return = 0;
 int s_StartPort, s_EndPort;
 t_MZ_PortScan *s_PortScan, *s_TracePortScan;
 struct servent *s_ServiceEntry;
 
 if(s_Argc > 1)
 {
  s_StartPort = (-1);
  s_EndPort = (-1);
  if(s_Argc > 2)
  {
   sscanf(s_Argv[2], "%i", &s_StartPort);
   if(s_Argc > 3)
   {
    sscanf(s_Argv[3], "%i", &s_EndPort);
   }
  }
  fprintf(stdout, "Port scanning \"%s\". . .\n\n", s_Argv[1]);
  s_PortScan = MZ_PortScan(s_Argv[1], s_StartPort, s_EndPort, 1);
  if(s_PortScan)
  { 
   fprintf(stdout, "%-5s %s\n", "Port", "Service"); 
   s_TracePortScan = s_PortScan;
   while(s_TracePortScan)
   {
    s_ServiceEntry = getservbyport(htons(s_TracePortScan->Port), "tcp");
    fprintf(stdout, "%5d %s\n", s_TracePortScan->Port, s_ServiceEntry ? s_ServiceEntry->s_name : "Unknown");
    s_TracePortScan = s_TracePortScan->Next;
   }
   s_PortScan = MZ_FreePortScan(s_PortScan);
  }
  else fprintf(stderr, "portscan: All closed.\n");
 }
 else fprintf(stdout, "Usage: portscan <host> [<start port> [<end port>]]\n");
 
 return(s_Return);
}
#endif

#endif

/* End of source */
