/* 
 Copyright (c) 2004 JaeHyuk Cho <mailto:minzkn@infoeq.com>
 All right reserved.
*/

/* 
 x86 machine's minz boot code
*/

DEF_READ_SECTORS = (65536 / 512)

.code16
.org 0x0000
L_Entry:
	cli
	movw %cs, %ax
	movw %ax, %ds
	callw 0f
0:
	popw %si
	subw $(0b - L_Entry), %si
	movw $0x0050, %ax
	movw %ax, %es
	xorw %di, %di
	cld
	movw $(512 / 2), %cx
	repz movsw %ds:(%si), %es:(%di)
	ljmp $0x0050, $0f
0:	
	movw %ax, %ds
	xorw %ax, %ax
	movw %ax, %ss
	xorw %sp, %sp

	/* Reset floppy disk controller */
	xorw %ax, %ax
	xorb %dl, %dl
	int $0x13

	/* Get disk parameter table */
	callw P_GetParameter 

	/* Read to SEG:OFF=1000H:0000H from sector 1 ~ */
	movw $0x1000, %ax
	movw %ax, %es 
	xorw %bx, %bx
	movw $0x0001, %ax
	xorw %dx, %dx
	movw $DEF_READ_SECTORS, %cx /* n sectors */
0:
	pushw %ax
	pushw %bx
	pushw %cx
	pushw %dx

	pushw %ax 
	callw P_Read /* ES:BX=Memory, DX:AX=LBA sector */
	popw %ax
	callw P_PrintDigit
	movw $L_ReadMessage_00, %si
	callw P_Puts
	movw $DEF_READ_SECTORS, %ax
	callw P_PrintDigit
	movw $L_ReadMessage_01, %si
	callw P_Puts
	movw (L_SectorsPerTrack), %ax
	callw P_PrintDigit
	movw $L_ReadMessage_02, %si
	callw P_Puts
	movw (L_Heads), %ax
	callw P_PrintDigit
	movw $L_ReadMessage_03, %si
	callw P_Puts

	popw %dx
	popw %cx
	popw %bx
	popw %ax
	addw $512, %bx
	addw $1, %ax
	adcw $0, %dx
	loop 0b

	/* Reset floppy disk controller */
	xorw %ax, %ax
	xorb %dl, %dl
	int $0x13

	movw $L_BootMessage, %si
	callw P_Puts

	ljmp $0x1000, $0x0000

	/* === Procedure === */

.align 2
P_PrintDigit: /* void P_PrintDigit(AX) */
	movw $10, %bx
	xorw %cx, %cx
0:
	xorw %dx, %dx
	divw %bx
	addw $0x30, %dx
	pushw %dx
	incw %cx
	orw %ax, %ax
	jnz 0b
0:
	popw %ax
	movb $0x0e, %ah
	movw $0x0700, %bx 
	int $0x10
	loop 0b
	retw

.align 2
P_GetParameter:
	xorw %ax, %ax
	movw %ax, %es
	movw $0x0078, %bx
	lesw %es:(%bx), %bx
	movb %es:0x04(%bx), %al
	movb %al, (L_SectorsPerTrack)
	retw

.align 2
P_LBS_TO_CHS: /* void P_LBS_TO_CHS(DX:AX) */
	divw (L_SectorsPerTrack)
	incb %dl
	movb %dl, (L_Sector)
	xorw %dx, %dx
	divw (L_Heads)
	movb %dl, (L_Head)
	movw %ax, (L_Track)
	retw

.align 2
P_ReadSector: /* void P_ReadSector(ES:BX) */
	movw $0x0201, %ax
	movw (L_Track), %cx
	shlb $6, %ch
	orb (L_Sector), %ch
	xchgb %ch, %cl
	movb (L_Drive), %dl
	movb (L_Head), %dh
	int $0x13
	jnc 1f
	movw $0f, %si
	callw P_Error
0:
	.string "Disk read error !\r\n"
1:	
	retw

.align 2
P_Read: /* void P_Read(ES:BX, DX:AX) */
	callw P_LBS_TO_CHS
	callw P_ReadSector
	retw

L_SectorsPerTrack:
	.word 0x0012 /* Default 18 */
L_Heads:
	.word 0x0002 /* Default 2 */
L_Drive:
	.byte 0x00
L_Sector:
	.byte 0x00
L_Head:
	.byte 0x00
L_Track:
	.word 0x0000
L_ReadMessage_00:
	.string "/"
L_ReadMessage_01:
	.string " sector read. (Sectors per track="
L_ReadMessage_02:
	.string ", Heads="
L_ReadMessage_03:
	.string ")\r\n"
L_BootMessage:
	.string "\r\nStarting MINZ. . .\r\n"

	/* === Procedure === */

.align 2 /* void P_Puts(%si) */
P_Puts:
	movb $0x0e, %ah
	movw $0x0700, %bx
0:
	lodsb
	orb %al, %al
	jz 0f
	int $0x10
	jmp 0b
0:
	retw

.align 2 /* %ax P_Puts(void) */
P_PressAnyKey:
	xorw %ax, %ax
	int $0x16
	retw

.align 2 /* reboot P_Error(%si) */
P_Error:
	callw P_Puts
	callw P_PressAnyKey
	int $0x18
	ljmp $0xffff, $0x0000
	retw

.org 0x1fe
	.word 0xaa55

.globl L_Entry

.end

/* End of source */
