/*
 Copyright (c) Information Equipment co.,LTD.
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.co.kr> KOREA

 - GNU GENERAL PUBLIC LICENSE
 - Simple is best !
*/

#if !defined(DEF_SOURCE_dump_c)
#define DEF_SOURCE_dump_c "dump.c"

#include <string.h>

#define __ML_DUMP_WIDTH__ ( 16 )
#define __ML_DUMP_CACHE__ ( 1 << 10 )
#define __ML_DUMP_COPY__  ML_CopyMemory

static int __ML_Dump_Write__(int s_Handle, const void *s_Data, int s_Size);
static int __ML_Dump_ConvertHexa__(void *s_String, int s_Value);
static void __ML_Dump_Put__(int s_Handle, const void *s_String, int s_Length);
static void __ML_Dump_Hexa__(int s_Handle, int s_Value, int s_Place);
static void __ML_Dump__(int s_Handle, int s_Offset, int s_Index, const void *s_Data, int s_Width);
void ML_Dump(int s_Handle, int s_Offset, const void *s_Data, int s_Size);

static int __ML_Dump_Write__(int s_Handle, const void *s_Data, int s_Size)
{
 register int s_Return;	
 __asm__ __volatile__(
  "\n\t"
  "int $0x80\n\t"
  "\n\t"
  : "=a"(s_Return)
  : "a"(4), "b"(s_Handle), "c"(s_Data), "d"(s_Size)
 );
 return(s_Return);
}

static int __ML_Dump_ConvertHexa__(void *s_String, int s_Value)
{
 const char c_HexaTable[] = "0123456789ABCDEF";
 char s_FILO[8];
 int s_Return, s_Index;
 for(s_Return = 0;s_Value > 0;s_FILO[s_Return++] = c_HexaTable[s_Value & 0x0f], s_Value >>= 4);
 for(s_Index = s_Return;s_Index > 0;*(((char *)s_String)++) = s_FILO[--s_Index]); 
 *((char *)s_String) = '\0';
 return(s_Return);
}

static void __ML_Dump_Put__(int s_Handle, const void *s_String, int s_Length)
{
 static char sg_Cache[ __ML_DUMP_CACHE__ ];
 static int  sg_Index = 0;
 if((s_Length == 0) || ((sg_Index + s_Length) > sizeof(sg_Cache)))
 {
  (void)__ML_Dump_Write__(s_Handle, &sg_Cache[0], sg_Index);
  sg_Index = 0;
 }
 if(s_Length > 0)
 {
  if(s_Length >= sizeof(sg_Cache))(void)__ML_Dump_Write__(1, s_String, s_Length);
  else
  { 
   memcpy((void *)(&sg_Cache[sg_Index]), s_String, s_Length);
   sg_Index += s_Length;
  }
 } 
}

static void __ML_Dump_Hexa__(int s_Handle, int s_Value, int s_Place)
{
 char s_Zero = '0', s_ConvertBuffer[8 + 1];
 int s_Length = __ML_Dump_ConvertHexa__(&s_ConvertBuffer[0], s_Value), s_Index;
 s_Index = s_Length;
 while((s_Index++) < s_Place)__ML_Dump_Put__(s_Handle, &s_Zero, sizeof(s_Zero));
 if(s_Length > 0)__ML_Dump_Put__(s_Handle, &s_ConvertBuffer[0], s_Length);
}

static void __ML_Dump__(int s_Handle, int s_Offset, int s_Index, const void *s_Data, int s_Width)
{
 const char c_More[] = {' ', '|'}, c_Space = ' ', c_Dot = '.', c_NewLine = '\n';
 const char c_SpaceTable[] = {' ', ' ', ' ', ' ', ' '};	
 int s_Count;
 char s_Right[ 1 + 1 + __ML_DUMP_WIDTH__ + 1 + 1 ];	
 s_Right[0] = c_Space, s_Right[1] = '[', s_Right[ sizeof(s_Right) - 2 ] = ']', s_Right[ sizeof(s_Right) - 1 ] = c_NewLine;
 __ML_Dump_Hexa__(s_Handle, s_Offset + s_Index, 8);
 for(s_Count = 0;s_Count < s_Width;s_Count++)
 {
  if(s_Count == (__ML_DUMP_WIDTH__ >> 1))__ML_Dump_Put__(s_Handle, &c_More[0], sizeof(c_More));
  __ML_Dump_Put__(s_Handle, &c_Space, sizeof(c_Space));
  s_Right[ s_Count + 2 ] = *(((unsigned char *)s_Data) + s_Count);
  __ML_Dump_Hexa__(s_Handle, s_Right[ s_Count + 2 ], 2);
  if(s_Right[ s_Count + 2 ] >= 0x7f || s_Right[ s_Count + 2 ] < c_Space)s_Right[ s_Count + 2 ] = c_Dot; 
 }
 for(;s_Count < __ML_DUMP_WIDTH__;s_Count++)
 { 
  if(s_Count == (__ML_DUMP_WIDTH__ >> 1))__ML_Dump_Put__(s_Handle, &c_SpaceTable[0], 2 + 3);
  else __ML_Dump_Put__(s_Handle, &c_SpaceTable[0], 3);
  s_Right[ s_Count + 2 ] = c_Space; 
 } 
 __ML_Dump_Put__(s_Handle, &s_Right[0], sizeof(s_Right));
}	

void ML_Dump(int s_Handle, int s_Offset, const void *s_Data, int s_Size)
{
 int s_Index = 0, s_Width;
 if(s_Handle < 0)s_Handle = 1;
 while(s_Index < s_Size)
 {
  s_Width = s_Size - s_Index; 
  if(s_Width > __ML_DUMP_WIDTH__)s_Width = __ML_DUMP_WIDTH__;
  __ML_Dump__(s_Handle, s_Offset, s_Index, (void *)(((unsigned char *)s_Data) + s_Index), s_Width);  
  s_Index += __ML_DUMP_WIDTH__;
 }
 __ML_Dump_Put__(s_Handle, (void *)0, 0); 
}

#endif

/* End of source */
